@@ -29,12 +29,13 @@
 #include "prefs.h"
 #include "setting.h"
 #include "browser.h"
-#include "scan.h"
+#include "scan_dialog.h"
 #include "cddb.h"
 #include "log.h"
 #include "misc.h"
 #include "charset.h"
 #include "ui_manager.h"
+#include "gtk2_compat.h"
 
 /***************
  * Declaration *
@@ -637,7 +638,7 @@ et_statusbar_pop_tooltip (void)
  */
 GtkWidget *Create_Progress_Bar (void)
 {
-    ProgressBar = gtk_progress_bar_new();
+    ProgressBar = et_progress_bar_new ();
 
     gtk_widget_show(ProgressBar);
     return ProgressBar;
