@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -24,7 +24,7 @@
 #ifdef ENABLE_FLAC
 
 #include <gtk/gtk.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -62,10 +62,13 @@
  *  - ARTIST       : Track performer
  *  - ORGANIZATION : Name of the organization producing the track (i.e. the 'record label')
  *  - DESCRIPTION  : A short text description of the contents
- *  - COMMENT      : same than DESCRIPTION
+ *  - COMMENT      : same as DESCRIPTION
  *  - GENRE        : A short text indication of music genre
  *  - DATE         : Date the track was recorded
  *  - LOCATION     : Location where track was recorded
+ *  - CONTACT      : Contact information for the creators or distributors of
+ *                   the track. This could be a URL, an email address, the
+ *                   physical address of the producing label.
  *  - COPYRIGHT    : Copyright information
  *  - ISRC         : ISRC number for the track; see the ISRC intro page for more information on ISRC numbers.
  *
@@ -104,9 +107,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     Picture *prev_pic = NULL;
     //gint j = 1;
 
-    
-    if (!filename || !FileTag)
-        return FALSE;
+    g_return_val_if_fail (filename != NULL && FileTag != NULL, FALSE);
 
     flac_error_msg = NULL;
 
@@ -271,9 +272,33 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                     }
                 }
 
-                /***************
-                 * Disc Number *
-                 ***************/
+                /******************************
+                 * Disc Number and Disc Total *
+                 ******************************/
+                if ((field_num = FLAC__metadata_object_vorbiscomment_find_entry_from (block, 0, "DISCTOTAL")) >= 0)
+                {
+                    /* Extract field value. */
+                    field = &vc->comments[field_num];
+                    field_value = memchr (field->entry, '=', field->length);
+
+                    if (field_value)
+                    {
+                        field_value++;
+                        if (field_value && g_utf8_strlen (field_value, -1) > 0)
+                        {
+                            field_len = field->length - (field_value - (gchar*) field->entry);
+                            field_value_tmp = g_strndup (field_value,
+                                                         field_len);
+                            field_value = Try_To_Validate_Utf8_String (field_value_tmp);
+                            g_free (field_value_tmp);
+
+                            FileTag->disc_total = et_disc_number_to_string (atoi (field_value));
+                            g_free (field_value);
+                        }
+                    }
+                    /* Discs is also filled below, if not done here. */
+                }
+
                 if ( (field_num = FLAC__metadata_object_vorbiscomment_find_entry_from(block,0,"DISCNUMBER")) >= 0 )
                 {
                     /* Extract field value */
@@ -289,7 +314,17 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                             field_value_tmp = g_strndup(field_value, field_len);
                             field_value = Try_To_Validate_Utf8_String(field_value_tmp);
                             g_free(field_value_tmp);
-                            FileTag->disc_number = field_value;
+                            string = g_utf8_strchr (field_value, -1, '/');
+
+                            if (string && !FileTag->disc_total)
+                            {
+                                FileTag->disc_total = et_disc_number_to_string (atoi (string + 1));
+                                *string = '\0';
+                            }
+
+                            FileTag->disc_number = et_disc_number_to_string (atoi (field_value));
+
+                            g_free (field_value);
                         }
                     }
                 }
@@ -337,13 +372,8 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                             field_value_tmp = g_strndup(field_value, field_len);
                             field_value = Try_To_Validate_Utf8_String(field_value_tmp);
                             g_free(field_value_tmp);
-                            if (NUMBER_TRACK_FORMATED)
-                            {
-                                FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(field_value));
-                            }else
-                            {
-                                FileTag->track_total = g_strdup(field_value);
-                            }
+
+                            FileTag->track_total = et_track_number_to_string (atoi (field_value));
                             g_free(field_value);
                         }
                     }
@@ -366,24 +396,14 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                             field_value = Try_To_Validate_Utf8_String(field_value_tmp);
                             g_free(field_value_tmp);
                             string = g_utf8_strchr(field_value, -1, '/');
-                            if (NUMBER_TRACK_FORMATED)
+
+                            if (string && !FileTag->track_total)
                             {
-                                // If track_total not filled before, try now...
-                                if (string && !FileTag->track_total)
-                                {
-                                    FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string+1));
-                                    *string = '\0';
-                                }
-                                FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(field_value));
-                            }else
-                            {
-                                if (string && !FileTag->track_total)
-                                {
-                                    FileTag->track_total = g_strdup(string+1);
-                                    *string = '\0';
-                                }
-                                FileTag->track = g_strdup(field_value);
+                                FileTag->track_total = et_track_number_to_string (atoi (string + 1));
+                                *string = '\0';
                             }
+                            FileTag->track = et_track_number_to_string (atoi (field_value));
+
                             g_free(field_value);
                         }
                     }
@@ -553,7 +573,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                  * URL *
                  *******/
                 field_num = 0;
-                while ( (field_num = FLAC__metadata_object_vorbiscomment_find_entry_from(block,field_num,"LICENSE")) >= 0 )
+                while ((field_num = FLAC__metadata_object_vorbiscomment_find_entry_from (block, field_num, "CONTACT")) >= 0)
                 {
                     /* Extract field value */
                     field = &vc->comments[field_num++];
@@ -616,6 +636,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                       && strncasecmp((gchar *)field->entry,"ALBUMARTIST=", MIN(12, field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"ALBUM=",       MIN(6,  field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"DISCNUMBER=",  MIN(11, field->length)) != 0
+                      && strncasecmp((gchar *)field->entry,"DISCTOTAL=",   MIN(10, field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"DATE=",        MIN(5,  field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"TRACKNUMBER=", MIN(12, field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"TRACKTOTAL=",  MIN(11, field->length)) != 0
@@ -625,7 +646,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                       && strncasecmp((gchar *)field->entry,"COMPOSER=",    MIN(9,  field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"PERFORMER=",   MIN(10, field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"COPYRIGHT=",   MIN(10, field->length)) != 0
-                      && strncasecmp((gchar *)field->entry,"LICENSE=",     MIN(8,  field->length)) != 0
+                      && strncasecmp((gchar *)field->entry,"CONTACT=",     MIN(8,  field->length)) != 0
                       && strncasecmp((gchar *)field->entry,"ENCODED-BY=",  MIN(11, field->length)) != 0 )
                     {
                         //g_print("custom %*s\n", field->length, field->entry);
@@ -667,7 +688,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                 //pic->height = p->height;
                 //pic->width  = p->width;
 
-                //g_print("Picture type : %s\n",FLAC__StreamMetadata_Picture_TypeString[p->type]);
+                //g_print("Image type : %s\n",FLAC__StreamMetadata_Picture_TypeString[p->type]);
                 //g_print("Mime    type : %s\n",p->mime_type);
 
                 break;
@@ -693,6 +714,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
       && FileTag->album_artist == NULL
       && FileTag->album       == NULL
       && FileTag->disc_number == NULL
+      && FileTag->disc_total == NULL
       && FileTag->year        == NULL
       && FileTag->track       == NULL
       && FileTag->track_total == NULL
@@ -714,6 +736,7 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
           || FileTag->album_artist != NULL
           || FileTag->album       != NULL
           || FileTag->disc_number != NULL
+          || FileTag->disc_total != NULL
           || FileTag->year        != NULL
           || FileTag->track       != NULL
           || FileTag->track_total != NULL
@@ -732,26 +755,6 @@ gboolean Flac_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
         g_free(filename_utf8);
         return rc;
     }
-
-    /* Part to get cover artist :
-     * If we have read the ID3 tag previously we don't arrive here (and we have
-     * the picture if it exists).
-     * Else the ID3 tag wasn't read (as there was data in FLAC tag) so we try
-     * to read it only to get the picture (not supported by the FLAC tag). */
-    /***if (WRITE_ID3_TAGS_IN_FLAC_FILE && FileTag->picture == NULL)
-    {
-        File_Tag *FileTag_tmp = ET_File_Tag_Item_New();
-        gint rc = Id3tag_Read_File_Tag(filename,FileTag_tmp);
-        if (rc && FileTag_tmp->picture)
-        {
-            // Copy picture to FileTag
-            FileTag->picture = Picture_Copy(FileTag_tmp->picture);
-        }
-
-        ET_Free_File_Tag_Item(FileTag_tmp);
-
-        return rc;
-    }***/
 #endif
 
     g_free(filename_utf8);
@@ -818,9 +821,7 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
     FLAC__StreamMetadata_VorbisComment_Entry vce_field_vendor_string; // To save vendor string
     gboolean vce_field_vendor_string_found = FALSE;
 
-    
-    if (!ETFile || !ETFile->FileTag)
-        return FALSE;
+    g_return_val_if_fail (ETFile != NULL && ETFile->FileTag != NULL, FALSE);
 
     FileTag       = (File_Tag *)ETFile->FileTag->data;
     filename      = ((File_Name *)ETFile->FileNameCur->data)->value;
@@ -912,7 +913,7 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
     //
     {
         FLAC__StreamMetadata *vc_block; // For vorbis comments
-        GList *list;
+        GList *l;
         
         // Allocate a block for Vorbis comments
         vc_block = FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT);
@@ -947,10 +948,11 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
          *********/
         Flac_Set_Tag(vc_block,"ALBUM=",FileTag->album,VORBIS_SPLIT_FIELD_ALBUM);
 
-        /***************
-         * Disc Number *
-         ***************/
-        Flac_Set_Tag(vc_block,"DISCNUMBER=",FileTag->disc_number,FALSE);
+        /******************************
+         * Disc Number and Disc Total *
+         ******************************/
+        Flac_Set_Tag (vc_block, "DISCNUMBER=", FileTag->disc_number, FALSE);
+        Flac_Set_Tag (vc_block, "DISCTOTAL=", FileTag->disc_total, FALSE);
 
         /********
          * Year *
@@ -971,9 +973,7 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
         /***********
          * Comment *
          ***********/
-        // We write the comment using the "both" format
         Flac_Set_Tag(vc_block,"DESCRIPTION=",FileTag->comment,VORBIS_SPLIT_FIELD_COMMENT);
-        Flac_Set_Tag(vc_block,"COMMENT=",FileTag->comment,VORBIS_SPLIT_FIELD_COMMENT);
 
         /************
          * Composer *
@@ -993,7 +993,7 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
         /*******
          * URL *
          *******/
-        Flac_Set_Tag(vc_block,"LICENSE=",FileTag->url,FALSE);
+        Flac_Set_Tag (vc_block, "CONTACT=", FileTag->url, FALSE);
 
         /**************
          * Encoded by *
@@ -1004,11 +1004,9 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
         /**************************
          * Set unsupported fields *
          **************************/
-        list = FileTag->other;
-        while (list)
+        for (l = FileTag->other; l != NULL; l = g_list_next (l))
         {
-            Flac_Set_Tag(vc_block, "", (gchar*)list->data, FALSE);
-            list = list->next;
+            Flac_Set_Tag (vc_block, "", (gchar *)l->data, FALSE);
         }
 
         // Add the block to the the chain (so we don't need to free the block)
@@ -1106,13 +1104,6 @@ gboolean Flac_Tag_Write_File_Tag (ET_File *ETFile)
 
     
 #ifdef ENABLE_MP3
-    /*
-     * Write also the ID3 tags (ID3v1 and/or ID3v2) if wanted (as needed by some players)
-     */
-    if (WRITE_ID3_TAGS_IN_FLAC_FILE)
-    {
-        Id3tag_Write_File_Tag(ETFile);
-    }else
     {
         // Delete the ID3 tags (create a dummy ETFile for the Id3tag_... function)
         ET_File   *ETFile_tmp    = ET_File_Item_New();
